@@ -1,14 +1,25 @@
 import torch
 from torch import nn, einsum
+import torch.nn.functional as F
 from einops import rearrange, repeat
 
 # classes
 
+class ScaleNorm(nn.Module):
+    def __init__(self, dim, eps = 1e-5):
+        super().__init__()
+        self.eps = eps
+        self.g = nn.Parameter(torch.ones(1))
+
+    def forward(self, x):
+        n = torch.norm(x, dim = -1, keepdim = True).clamp(min = self.eps)
+        return x / n * self.g
+
 class PreNorm(nn.Module):
     def __init__(self, dim, fn):
         super().__init__()
         self.fn = fn
-        self.norm = nn.LayerNorm(dim)
+        self.norm = ScaleNorm(dim)
 
     def forward(self, x, *args, **kwargs):
         x = self.norm(x)
@@ -16,12 +27,17 @@ class PreNorm(nn.Module):
 
 # feedforward
 
+class GEGLU(nn.Module):
+    def forward(self, x):
+        x, gates = x.chunk(2, dim = -1)
+        return x * F.gelu(gates)
+
 class FeedForward(nn.Module):
     def __init__(self, dim, mult = 4, dropout = 0.):
         super().__init__()
         self.net = nn.Sequential(
-            nn.Linear(dim, dim * mult),
-            nn.GELU(),
+            nn.Linear(dim, dim * mult * 2),
+            GEGLU(),
             nn.Dropout(dropout),
             nn.Linear(dim * mult, dim)
         )
@@ -133,7 +149,7 @@ class TimeSformer(nn.Module):
             ]))
 
         self.to_out = nn.Sequential(
-            nn.LayerNorm(dim),
+            ScaleNorm(dim),
             nn.Linear(dim, num_classes)
         )
 
@@ -155,5 +171,5 @@ class TimeSformer(nn.Module):
             x = spatial_attn(x, 'b (f n) d', '(b f) n d', f = f) + x
             x = ff(x) + x
 
-        cls_token = tokens[:, 0]
+        cls_token = x[:, 0]
         return self.to_out(cls_token)
